# -*- coding: utf-8 -*- #
# Copyright 2016 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for spanner instances list."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import textwrap

from googlecloudsdk.api_lib.spanner import instances
from googlecloudsdk.calliope import base


class List(base.ListCommand):
  """List the Cloud Spanner instances in this project."""

  detailed_help = {
      'EXAMPLES':
          textwrap.dedent("""\
        To list all Cloud Spanner instances in this project, run:

          $ {command}
        """),
  }

  @staticmethod
  def Args(parser):
    parser.display_info.AddFormat("""
          table(
            name.basename(),
            displayName,
            config.basename(),
            nodeCount,
            processing_units,
            state,
            instance_type
          )
        """)

  def Run(self, args):
    """This is what gets called when the user runs this command.

    Args:
      args: an argparse namespace. All the arguments that were provided to this
        command invocation.

    Returns:
      Some value that we want to have printed later.
    """
    return instances.List()



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""VMware Engine external access rules client."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import re

from apitools.base.py import list_pager
from googlecloudsdk.api_lib.vmware import util


class ExternalAccessRulesClient(util.VmwareClientBase):
  """VMware Engine network policy client."""

  def __init__(self):
    super(ExternalAccessRulesClient, self).__init__()
    self.service = self.client.projects_locations_networkPolicies_externalAccessRules
    self.ip_regex = re.compile(r'\d{1,3}.\d{1,3}.\d{1,3}.\d{1,3}$')
    self.ip_ranges_regex = re.compile(
        r'\d{1,3}.\d{1,3}.\d{1,3}.\d{1,3}/\d{1,2}$')

  def parse_ip_range(self, ip_range):
    if self.ip_regex.match(ip_range) is not None:
      return self.messages.IpRange(ipAddress=ip_range)
    if self.ip_ranges_regex.match(ip_range) is not None:
      return self.messages.IpRange(ipAddressRange=ip_range)
    return self.messages.IpRange(externalAddress=ip_range)

  def Get(self, resource):
    request = self.messages.VmwareengineProjectsLocationsNetworkPoliciesExternalAccessRulesGetRequest(
        name=resource.RelativeName())
    response = self.service.Get(request)
    return response

  def Create(self,
             resource,
             priority,
             ip_protocol,
             source_ranges,
             destination_ranges,
             source_ports,
             destination_ports,
             description=None,
             action=None):
    parent = resource.Parent().RelativeName()
    external_access_rule_id = resource.Name()
    external_access_rule = self.messages.ExternalAccessRule(
        description=description,
        priority=priority,
        ipProtocol=ip_protocol)

    if source_ports is None:
      external_access_rule.sourcePorts = []
    else:
      external_access_rule.sourcePorts = source_ports

    if destination_ports is None:
      external_access_rule.destinationPorts = []
    else:
      external_access_rule.destinationPorts = destination_ports

    if action is None or action.strip().upper() == 'ALLOW':
      external_access_rule.action = self.messages.ExternalAccessRule.ActionValueValuesEnum.ALLOW
    elif action.strip().upper() == 'DENY':
      external_access_rule.action = self.messages.ExternalAccessRule.ActionValueValuesEnum.DENY

    external_access_rule.sourceIpRanges = [
        self.parse_ip_range(ip) for ip in source_ranges
    ]
    external_access_rule.destinationIpRanges = [
        self.parse_ip_range(ip) for ip in destination_ranges
    ]
    request = self.messages.VmwareengineProjectsLocationsNetworkPoliciesExternalAccessRulesCreateRequest(
        parent=parent,
        externalAccessRule=external_access_rule,
        externalAccessRuleId=external_access_rule_id,
    )
    return self.service.Create(request)

  def Update(self,
             resource,
             priority=None,
             ip_protocol=None,
             source_ranges=None,
             destination_ranges=None,
             source_ports=None,
             destination_ports=None,
             description=None,
             action=None):
    external_access_rule = self.Get(resource)
    update_mask = []
    if description is not None:
      external_access_rule.description = description
      update_mask.append('description')
    if priority is not None:
      external_access_rule.priority = priority
      update_mask.append('priority')
    if ip_protocol is not None:
      external_access_rule.ipProtocol = ip_protocol
      update_mask.append('ip_protocol')
    if source_ports is not None:
      external_access_rule.sourcePorts = source_ports
      update_mask.append('source_ports')
    if destination_ports is not None:
      external_access_rule.destinationPorts = destination_ports
      update_mask.append('destination_ports')

    if action is not None:
      if action.strip().upper() == 'ALLOW':
        external_access_rule.action = self.messages.ExternalAccessRule.ActionValueValuesEnum.ALLOW
      elif action.strip().upper() == 'DENY':
        external_access_rule.action = self.messages.ExternalAccessRule.ActionValueValuesEnum.DENY
      update_mask.append('action')

    if source_ranges is not None and source_ranges:
      external_access_rule.sourceIpRanges = [
          self.parse_ip_range(ip) for ip in source_ranges
      ]
      update_mask.append('source_ip_ranges')
    if destination_ranges is not None and destination_ranges:
      external_access_rule.destinationIpRanges = [
          self.parse_ip_range(ip) for ip in destination_ranges
      ]
      update_mask.append('destination_ip_ranges')
    request = self.messages.VmwareengineProjectsLocationsNetworkPoliciesExternalAccessRulesPatchRequest(
        externalAccessRule=external_access_rule,
        name=resource.RelativeName(),
        updateMask=','.join(update_mask),
    )
    return self.service.Patch(request)

  def Delete(self, resource):
    return self.service.Delete(
        self.messages.VmwareengineProjectsLocationsNetworkPoliciesExternalAccessRulesDeleteRequest(
            name=resource.RelativeName()
        )
    )

  def List(self, network_policy_resource):
    network_policy = network_policy_resource.RelativeName()
    request = self.messages.VmwareengineProjectsLocationsNetworkPoliciesExternalAccessRulesListRequest(
        parent=network_policy
    )
    return list_pager.YieldFromList(
        self.service,
        request,
        batch_size_attribute='pageSize',
        field='externalAccessRules')



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to import rules in a conversion workspaces for a database migration from config files."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.database_migration import conversion_workspaces
from googlecloudsdk.api_lib.database_migration import resource_args
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.database_migration.conversion_workspaces import flags as cw_flags
from googlecloudsdk.core import log


@base.Hidden
@base.ReleaseTracks(base.ReleaseTrack.GA)
class ImportRules(base.Command):
  """Import mapping rules in a Database Migration Service conversion workspace."""

  detailed_help = {
      'DESCRIPTION': """
        Import rules in a Database Migration Service conversion workspace.
        """,
      'EXAMPLES': """\
        To import rules in a conversion workspace:

            $ {command} my-conversion-workspace --region=us-central1
            --file-format=ORA2PG --config-files=PATH1/config1.conf,PATH2/config2.conf
        """,
  }

  @staticmethod
  def Args(parser):
    """Args is called by calliope to gather arguments for this command.

    Args:
      parser: An argparse parser that you can use to add arguments that go on
        the command line after this command. Positional arguments are allowed.
    """
    resource_args.AddConversionWorkspaceResourceArg(parser, 'to import rules')
    cw_flags.AddImportFileFormatFlag(parser)
    cw_flags.AddConfigFilesFlag(parser)
    cw_flags.AddAutoCommitFlag(parser)

  def Run(self, args):
    """Import rules in a Database Migration Service conversion workspace.

    Args:
      args: argparse.Namespace, The arguments that this command was invoked
        with.
    """
    conversion_workspace_ref = args.CONCEPTS.conversion_workspace.Parse()
    cw_client = conversion_workspaces.ConversionWorkspacesClient(
        self.ReleaseTrack()
    )
    # Import mapping rules is a passthrough method and returns only done: true
    # along with the error (if any)
    result_operation = cw_client.ImportRules(
        conversion_workspace_ref.RelativeName(), args
    )

    if result_operation.error is not None:
      log.status.Print(
          'Imported mapping rules for conversion workspace {} failed with error'
          ' [{}]'.format(
              conversion_workspace_ref.conversionWorkspacesId,
              result_operation.error.message,
          )
      )
    else:
      log.status.Print(
          'Imported mapping rules for conversion workspace {}'.format(
              conversion_workspace_ref.conversionWorkspacesId
          )
      )



# -*- coding: utf-8 -*- #
# Copyright 2015 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Create cluster command."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.dataproc import compute_helpers
from googlecloudsdk.api_lib.dataproc import constants
from googlecloudsdk.api_lib.dataproc import dataproc as dp
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.command_lib.dataproc import clusters
from googlecloudsdk.command_lib.dataproc import flags
from googlecloudsdk.command_lib.kms import resource_args as kms_resource_args
from googlecloudsdk.command_lib.util.apis import arg_utils
from googlecloudsdk.command_lib.util.args import labels_util


@base.ReleaseTracks(base.ReleaseTrack.GA)
class Create(base.CreateCommand):
  """Create a cluster."""

  # DEPRECATED Beta release track should no longer be used, Google Cloud
  # no longer supports it.
  BETA = False

  detailed_help = {
      'EXAMPLES': """\
          To create a cluster, run:

            $ {command} my-cluster --region=us-central1
      """
  }

  @classmethod
  def Args(cls, parser):
    dataproc = dp.Dataproc(cls.ReleaseTrack())
    base.ASYNC_FLAG.AddToParser(parser)
    flags.AddClusterResourceArg(parser, 'create', dataproc.api_version)
    clusters.ArgsForClusterRef(
        parser,
        dataproc,
        cls.BETA,
        include_ttl_config=True,
        include_gke_platform_args=cls.BETA,
        include_driver_pool_args=True)
    # Add arguments for failure action for primary workers
    if not cls.BETA:
      parser.add_argument(
          '--action-on-failed-primary-workers',
          choices={
              'NO_ACTION': 'take no action',
              'DELETE': 'delete the failed primary workers',
              'FAILURE_ACTION_UNSPECIFIED': 'failure action is not specified'
          },
          type=arg_utils.ChoiceToEnumName,
          help="""
        Failure action to take when primary workers fail during cluster creation
        """)
    # Add gce-pd-kms-key args
    kms_flag_overrides = {
        'kms-key': '--gce-pd-kms-key',
        'kms-keyring': '--gce-pd-kms-key-keyring',
        'kms-location': '--gce-pd-kms-key-location',
        'kms-project': '--gce-pd-kms-key-project'
    }
    kms_resource_args.AddKmsKeyResourceArg(
        parser, 'cluster', flag_overrides=kms_flag_overrides)

  @staticmethod
  def ValidateArgs(args):
    if constants.ALLOW_ZERO_WORKERS_PROPERTY in args.properties:
      raise exceptions.InvalidArgumentException(
          '--properties',
          'Instead of %s, use gcloud beta dataproc clusters create '
          '--single-node to deploy single node clusters' %
          constants.ALLOW_ZERO_WORKERS_PROPERTY)

    clusters.ValidateReservationAffinityGroup(args)

  def Run(self, args):
    self.ValidateArgs(args)

    dataproc = dp.Dataproc(self.ReleaseTrack())

    cluster_ref = args.CONCEPTS.cluster.Parse()

    compute_resources = compute_helpers.GetComputeResources(
        self.ReleaseTrack(), cluster_ref.clusterName, cluster_ref.region)

    cluster_config = clusters.GetClusterConfig(
        args,
        dataproc,
        cluster_ref.projectId,
        compute_resources,
        self.BETA,
        include_ttl_config=True,
        include_gke_platform_args=self.BETA)

    action_on_failed_primary_workers = None
    if not self.BETA:
      action_on_failed_primary_workers = arg_utils.ChoiceToEnum(
          args.action_on_failed_primary_workers,
          dataproc.messages.DataprocProjectsRegionsClustersCreateRequest
          .ActionOnFailedPrimaryWorkersValueValuesEnum)

    cluster = dataproc.messages.Cluster(
        config=cluster_config,
        clusterName=cluster_ref.clusterName,
        projectId=cluster_ref.projectId)

    self.ConfigureCluster(dataproc.messages, args, cluster)

    return clusters.CreateCluster(
        dataproc,
        cluster_ref,
        cluster,
        args.async_,
        args.timeout,
        enable_create_on_gke=self.BETA,
        action_on_failed_primary_workers=action_on_failed_primary_workers)

  @staticmethod
  def ConfigureCluster(messages, args, cluster):
    """Performs any additional configuration of the cluster."""
    cluster.labels = labels_util.ParseCreateArgs(args,
                                                 messages.Cluster.LabelsValue)


# DEPRECATED Beta & Alpha release tracks should no longer be used, Google Cloud
# no longer supports them.
@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA)
class CreateBeta(Create):
  """Create a cluster."""
  BETA = True

  @classmethod
  def Args(cls, parser):
    super(CreateBeta, cls).Args(parser)
    clusters.BetaArgsForClusterRef(parser)

  @staticmethod
  def ValidateArgs(args):
    super(CreateBeta, CreateBeta).ValidateArgs(args)
    if args.master_accelerator and 'type' not in args.master_accelerator:
      raise exceptions.InvalidArgumentException(
          '--master-accelerator', 'accelerator type must be specified. '
          'e.g. --master-accelerator type=nvidia-tesla-k80,count=2')
    if args.worker_accelerator and 'type' not in args.worker_accelerator:
      raise exceptions.InvalidArgumentException(
          '--worker-accelerator', 'accelerator type must be specified. '
          'e.g. --worker-accelerator type=nvidia-tesla-k80,count=2')



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for creating instances."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute import filter_rewrite
from googlecloudsdk.api_lib.compute import utils
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.command_lib.compute import scope as compute_scopes
from googlecloudsdk.command_lib.compute.instances import flags as instances_flags
from googlecloudsdk.command_lib.compute.instances.bulk import flags as bulk_flags
from googlecloudsdk.command_lib.compute.instances.bulk import util as bulk_util
from googlecloudsdk.core import log
from googlecloudsdk.core import properties

DETAILED_HELP = {
    'brief':
        """
          Create multiple Compute Engine virtual machines.
        """,
    'DESCRIPTION':
        """
        *{command}* facilitates the creation of multiple Compute Engine
        virtual machines with a single command. They offer a number of advantages
        compared to the single instance creation command. This includes the
        ability to automatically pick a zone in which to create instances based
        on resource availability, the ability to specify that the request be
        atomic or best-effort, and a faster rate of instance creation.
        """,
    'EXAMPLES':
        """
        To create instances called 'example-instance-1', 'example-instance-2',
        and 'example-instance-3' in the 'us-central1-a' zone, run:

          $ {command} --predefined-names=example-instance-1,example-instance-2,example-instance-3 --zone=us-central1-a
        """,
}


def _GetOperations(compute_client, project, operation_group_id):
  """Requests operations with group id matching the given one."""

  errors_to_collect = []

  _, operation_filter = filter_rewrite.Rewriter().Rewrite(
      expression='operationGroupId=' + operation_group_id)

  operations_response = compute_client.MakeRequests(
      [(compute_client.apitools_client.globalOperations, 'AggregatedList',
        compute_client.apitools_client.globalOperations.GetRequestType(
            'AggregatedList')(filter=operation_filter, project=project))],
      errors_to_collect=errors_to_collect,
      log_result=False,
      always_return_operation=True,
      no_followup=True)

  return operations_response, errors_to_collect


def _GetResult(compute_client, request, operation_group_id):
  """Requests operations with group id and parses them as an output."""

  operations_response, errors = _GetOperations(compute_client, request.project,
                                               operation_group_id)
  if errors:
    utils.RaiseToolException(errors, error_message='Could not fetch resource:')
  result = {'operationGroupId': operation_group_id, 'instances': []}

  successful = [
      op for op in operations_response if op.operationType == 'insert' and
      str(op.status) == 'DONE' and op.error is None
  ]
  num_successful = len(successful)
  num_unsuccessful = request.bulkInsertInstanceResource.count - num_successful

  def GetInstanceStatus(op):
    return {
        'id': op.targetId,
        'name': op.targetLink.split('/')[-1],
        'zone': op.zone,
        'selfLink': op.targetLink
    }

  instances_status = [GetInstanceStatus(op) for op in successful]

  result['createdInstanceCount'] = num_successful
  result['failedInstanceCount'] = num_unsuccessful
  result['instances'] = instances_status

  return result


@base.ReleaseTracks(base.ReleaseTrack.GA)
class Create(base.Command):
  """Create Compute Engine virtual machine instances."""

  _support_nvdimm = False
  _support_public_dns = False
  _support_erase_vss = True
  _support_min_node_cpu = True
  _support_source_snapshot_csek = False
  _support_image_csek = True
  _support_confidential_compute = True
  _support_post_key_revocation_action_type = True
  _support_rsa_encrypted = True
  _deprecate_maintenance_policy = True
  _support_create_disk_snapshots = True
  _support_boot_snapshot_uri = True
  _support_display_device = False
  _support_local_ssd_size = False
  _support_secure_tags = False
  _support_host_error_timeout_seconds = False
  _support_numa_node_count = False
  _support_visible_core_count = True
  _support_max_run_duration = False
  _support_enable_target_shape = True
  _support_confidential_compute_type = False
  _support_provisioned_throughput = False
  _support_no_address_in_networking = False
  _support_max_count_per_zone = False
  _support_local_ssd_recovery_timeout = False
  _support_network_queue_count = False

  _log_async = False

  @classmethod
  def Args(cls, parser):
    bulk_flags.AddCommonBulkInsertArgs(
        parser,
        base.ReleaseTrack.GA,
        deprecate_maintenance_policy=cls._deprecate_maintenance_policy,
        support_min_node_cpu=cls._support_min_node_cpu,
        support_erase_vss=cls._support_erase_vss,
        snapshot_csek=cls._support_source_snapshot_csek,
        image_csek=cls._support_image_csek,
        support_display_device=cls._support_display_device,
        support_local_ssd_size=cls._support_local_ssd_size,
        support_numa_node_count=cls._support_numa_node_count,
        support_visible_core_count=cls._support_visible_core_count,
        support_max_run_duration=cls._support_max_run_duration,
        support_enable_target_shape=cls._support_enable_target_shape,
        support_confidential_compute_type=cls._support_confidential_compute_type,
        support_provisioned_throughput=cls._support_provisioned_throughput,
        support_no_address_in_networking=cls._support_no_address_in_networking,
        support_max_count_per_zone=cls._support_max_count_per_zone,
    )
    cls.AddSourceInstanceTemplate(parser)

  # LINT.IfChange(instance_template)
  @classmethod
  def AddSourceInstanceTemplate(cls, parser):
    cls.SOURCE_INSTANCE_TEMPLATE = (
        bulk_flags.MakeBulkSourceInstanceTemplateArg())
    cls.SOURCE_INSTANCE_TEMPLATE.AddArgument(parser)

  # LINT.ThenChange(../../queued_resources/create.py:instance_template)

  def Collection(self):
    return 'compute.instances'

  def _CreateRequests(self, args, holder, compute_client, resource_parser,
                      project, location, scope):
    supported_features = bulk_util.SupportedFeatures(
        self._support_nvdimm,
        self._support_public_dns,
        self._support_erase_vss,
        self._support_min_node_cpu,
        self._support_source_snapshot_csek,
        self._support_image_csek,
        self._support_confidential_compute,
        self._support_post_key_revocation_action_type,
        self._support_rsa_encrypted,
        self._deprecate_maintenance_policy,
        self._support_create_disk_snapshots,
        self._support_boot_snapshot_uri,
        self._support_display_device,
        self._support_local_ssd_size,
        self._support_secure_tags,
        self._support_host_error_timeout_seconds,
        self._support_numa_node_count,
        self._support_visible_core_count,
        self._support_max_run_duration,
        self._support_local_ssd_recovery_timeout,
        self._support_enable_target_shape,
        self._support_confidential_compute_type,
        self._support_provisioned_throughput,
        self._support_max_count_per_zone,
    )
    bulk_instance_resource = bulk_util.CreateBulkInsertInstanceResource(
        args,
        holder,
        compute_client,
        resource_parser,
        project,
        location,
        scope,
        self.SOURCE_INSTANCE_TEMPLATE,
        supported_features,
    )

    if scope == compute_scopes.ScopeEnum.ZONE:
      instance_service = compute_client.apitools_client.instances
      request_message = (
          compute_client.messages.ComputeInstancesBulkInsertRequest(
              bulkInsertInstanceResource=bulk_instance_resource,
              project=project,
              zone=location,
          )
      )
    elif scope == compute_scopes.ScopeEnum.REGION:
      instance_service = compute_client.apitools_client.regionInstances
      request_message = (
          compute_client.messages.ComputeRegionInstancesBulkInsertRequest(
              bulkInsertInstanceResource=bulk_instance_resource,
              project=project,
              region=location,
          )
      )

    return instance_service, request_message

  def Run(self, args):
    """Runs bulk create command.

    Args:
      args: argparse.Namespace, An object that contains the values for the
        arguments specified in the .Args() method.

    Returns:
      A resource object dispatched by display.Displayer().
    """
    bulk_flags.ValidateBulkInsertArgs(
        args,
        support_enable_target_shape=self._support_enable_target_shape,
        support_max_run_duration=self._support_max_run_duration,
        support_image_csek=self._support_image_csek,
        support_source_snapshot_csek=self._support_source_snapshot_csek,
        support_max_count_per_zone=self._support_max_count_per_zone,
    )

    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    compute_client = holder.client
    resource_parser = holder.resources

    project = properties.VALUES.core.project.GetOrFail()
    location = None
    scope = None

    if args.IsSpecified('zone'):
      location = args.zone
      scope = compute_scopes.ScopeEnum.ZONE
    elif args.IsSpecified('region'):
      location = args.region
      scope = compute_scopes.ScopeEnum.REGION

    instances_service, request = self._CreateRequests(args, holder,
                                                      compute_client,
                                                      resource_parser, project,
                                                      location, scope)

    self._errors = []
    self._log_async = False
    self._status_message = None

    if args.async_:
      self._log_async = True
      try:
        response = instances_service.BulkInsert(request)
        self._operation_selflink = response.selfLink
        return {'operationGroupId': response.operationGroupId}
      except exceptions.HttpException as error:
        raise error

    errors_to_collect = []
    response = compute_client.MakeRequests(
        [(instances_service, 'BulkInsert', request)],
        errors_to_collect=errors_to_collect,
        log_result=False,
        always_return_operation=True,
        no_followup=True)

    self._errors = errors_to_collect
    if response:
      operation_group_id = response[0].operationGroupId
      result = _GetResult(compute_client, request, operation_group_id)
      if (result.get('createdInstanceCount') is not None and
          result.get('failedInstanceCount') is not None):
        self._status_message = 'VM instances created: {}, failed: {}.'.format(
            result['createdInstanceCount'], result['failedInstanceCount'])
      return result
    return

  def Epilog(self, resources_were_displayed):
    del resources_were_displayed
    if self._errors:
      log.error(self._errors[0][1])
    elif self._log_async:
      log.status.Print('Bulk instance creation in progress: {}'.format(
          self._operation_selflink))
    else:
      if self._errors:
        log.warning(self._errors[0][1])
      log.status.Print(
          'Bulk create request finished with status message: [{}]'.format(
              self._status_message))


@base.ReleaseTracks(base.ReleaseTrack.BETA)
class CreateBeta(Create):
  """Create Compute Engine virtual machine instances."""

  _support_display_device = True
  _support_secure_tags = False
  _support_host_error_timeout_seconds = True
  _support_numa_node_count = False
  _support_visible_core_count = True
  _support_max_run_duration = True
  _support_enable_target_shape = True
  _support_provisioned_throughput = False
  _support_no_address_in_networking = False
  _support_max_count_per_zone = False
  _support_local_ssd_recovery_timeout = False
  _support_network_queue_count = False

  @classmethod
  def Args(cls, parser):
    bulk_flags.AddCommonBulkInsertArgs(
        parser,
        base.ReleaseTrack.BETA,
        deprecate_maintenance_policy=cls._deprecate_maintenance_policy,
        support_min_node_cpu=cls._support_min_node_cpu,
        support_erase_vss=cls._support_erase_vss,
        snapshot_csek=cls._support_source_snapshot_csek,
        image_csek=cls._support_image_csek,
        support_display_device=cls._support_display_device,
        support_local_ssd_size=cls._support_local_ssd_size,
        support_numa_node_count=cls._support_numa_node_count,
        support_visible_core_count=cls._support_visible_core_count,
        support_max_run_duration=cls._support_max_run_duration,
        support_enable_target_shape=cls._support_enable_target_shape,
        support_provisioned_throughput=cls._support_provisioned_throughput,
        support_no_address_in_networking=cls._support_no_address_in_networking,
        support_max_count_per_zone=cls._support_max_count_per_zone,
    )
    cls.AddSourceInstanceTemplate(parser)

    # Flags specific to Beta release track
    instances_flags.AddHostErrorTimeoutSecondsArgs(parser)


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class CreateAlpha(Create):
  """Create Compute Engine virtual machine instances."""

  # LINT.IfChange(alpha_spec)
  _support_display_device = True
  _support_local_ssd_size = True
  _support_secure_tags = True
  _support_host_error_timeout_seconds = True
  _support_numa_node_count = True
  _support_visible_core_count = True
  _support_max_run_duration = True
  _support_enable_target_shape = True
  _support_confidential_compute_type = True
  _support_provisioned_throughput = True
  _support_no_address_in_networking = True
  _support_max_count_per_zone = True
  _support_local_ssd_recovery_timeout = True
  _support_network_queue_count = True

  @classmethod
  def Args(cls, parser):
    bulk_flags.AddCommonBulkInsertArgs(
        parser,
        base.ReleaseTrack.ALPHA,
        deprecate_maintenance_policy=cls._deprecate_maintenance_policy,
        support_min_node_cpu=cls._support_min_node_cpu,
        support_erase_vss=cls._support_erase_vss,
        snapshot_csek=cls._support_source_snapshot_csek,
        image_csek=cls._support_image_csek,
        support_display_device=cls._support_display_device,
        support_local_ssd_size=cls._support_local_ssd_size,
        support_numa_node_count=cls._support_numa_node_count,
        support_visible_core_count=cls._support_visible_core_count,
        support_max_run_duration=cls._support_max_run_duration,
        support_enable_target_shape=cls._support_enable_target_shape,
        support_confidential_compute_type=cls._support_confidential_compute_type,
        support_provisioned_throughput=cls._support_provisioned_throughput,
        support_no_address_in_networking=cls._support_no_address_in_networking,
        support_max_count_per_zone=cls._support_max_count_per_zone,
        support_network_queue_count=cls._support_network_queue_count,
    )

    cls.AddSourceInstanceTemplate(parser)

    # Flags specific to Alpha release track
    instances_flags.AddSecureTagsArgs(parser)
    instances_flags.AddHostErrorTimeoutSecondsArgs(parser)
    instances_flags.AddMaintenanceInterval().AddToParser(parser)
    instances_flags.AddLocalSsdRecoveryTimeoutArgs(parser)
  # LINT.ThenChange(../../queued_resources/create.py:alpha_spec)


Create.detailed_help = DETAILED_HELP



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The command to describe the status of the Config Management Feature."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.container.fleet import util
from googlecloudsdk.calliope import base as calliope_base
from googlecloudsdk.command_lib.container.fleet import resources
from googlecloudsdk.command_lib.container.fleet.config_management import utils
from googlecloudsdk.command_lib.container.fleet.features import base
from googlecloudsdk.core import log
from googlecloudsdk.core import yaml
from googlecloudsdk.core.util import semver


@calliope_base.ReleaseTracks(calliope_base.ReleaseTrack.ALPHA)
class Fetch(base.DescribeCommand):
  """Prints the Config Management configuration applied to the given membership.

  The output is in the format that is used by the apply subcommand. The fields
  that have not been configured will be shown with default values.

  ## EXAMPLES

  To fetch the applied Config Management configuration, run:

    $ {command}

  """

  feature_name = 'configmanagement'

  @classmethod
  def Args(cls, parser):
    resources.AddMembershipResourceArg(parser)

  def Run(self, args):
    membership = base.ParseMembership(
        args, prompt=True, autoselect=True, search=True)

    f = self.GetFeature()
    membership_spec = None
    version = utils.get_backfill_version_from_feature(f, membership)
    for full_name, spec in self.hubclient.ToPyDict(f.membershipSpecs).items():
      if util.MembershipPartialName(full_name) == util.MembershipPartialName(
          membership) and spec is not None:
        membership_spec = spec.configmanagement
    if membership_spec is None:
      log.status.Print('Membership {} not initialized'.format(membership))

    # load the config template and merge with config has been applied to the
    # feature spec
    template = yaml.load(utils.APPLY_SPEC_VERSION_1)
    full_config = template['spec']
    merge_config_sync(membership_spec, full_config, version)
    merge_policy_controller(membership_spec, full_config, version)
    merge_hierarchy_controller(membership_spec, full_config)

    return template


def merge_config_sync(spec, config, version):
  """Merge configSync set in feature spec with the config template.

  ConfigSync has nested object structs need to be flatten.

  Args:
    spec: the ConfigManagementMembershipSpec message
    config: the dict loaded from full config template
    version: the version string of the membership
  """
  if not spec or not spec.configSync:
    return
  cs = config[utils.CONFIG_SYNC]
  git = spec.configSync.git
  oci = spec.configSync.oci
  if spec.configSync.enabled is not None:
    cs['enabled'] = spec.configSync.enabled
  else:
    # when enabled is no set in feature spec, it's determined by syncRepo
    if (git and git.syncRepo) or (oci and oci.syncRepo):
      cs['enabled'] = True
  if spec.configSync.sourceFormat:
    cs['sourceFormat'] = spec.configSync.sourceFormat
  if (not version or
      semver.SemVer(version) >= semver.SemVer(utils.PREVENT_DRIFT_VERSION)):
    if spec.configSync.preventDrift:
      cs['preventDrift'] = spec.configSync.preventDrift
  else:
    del cs['preventDrift']

  if not git and not oci:
    return
  # Update sourceType if version >= 1.12.0
  if (not version or
      semver.SemVer(version) >= semver.SemVer(utils.OCI_SUPPORT_VERSION)):
    if git:
      cs['sourceType'] = 'git'
    elif oci:
      cs['sourceType'] = 'oci'
  else:
    del cs['sourceType']

  if cs['sourceType'] and cs['sourceType'] == 'oci':
    if oci.syncWaitSecs:
      cs['syncWait'] = oci.syncWaitSecs
    for field in [
        'policyDir', 'secretType', 'syncRepo', 'gcpServiceAccountEmail'
    ]:
      if hasattr(oci, field) and getattr(oci, field) is not None:
        cs[field] = getattr(oci, field)
  else:
    if git.syncWaitSecs:
      cs['syncWait'] = git.syncWaitSecs
    for field in [
        'policyDir', 'httpsProxy', 'secretType', 'syncBranch', 'syncRepo',
        'syncRev', 'gcpServiceAccountEmail'
    ]:
      if hasattr(git, field) and getattr(git, field) is not None:
        cs[field] = getattr(git, field)


def merge_policy_controller(spec, config, version):
  """Merge configSync set in feature spec with the config template.

  ConfigSync has nested object structs need to be flatten.

  Args:
    spec: the ConfigManagementMembershipSpec message
    config: the dict loaded from full config template
    version: the version string of the membership
  """
  if not spec or not spec.policyController:
    return
  c = config[utils.POLICY_CONTROLLER]
  for field in list(config[utils.POLICY_CONTROLLER]):
    if hasattr(spec.policyController, field) and getattr(
        spec.policyController, field) is not None:
      c[field] = getattr(spec.policyController, field)

  valid_version = not version or semver.SemVer(version) >= semver.SemVer(
      utils.MONITORING_VERSION)
  spec_monitoring = spec.policyController.monitoring
  if not valid_version:
    c.pop('monitoring', None)
  elif spec_monitoring:
    c['monitoring'] = spec_monitoring


def merge_hierarchy_controller(spec, config):
  if not spec or not spec.hierarchyController:
    return
  c = config[utils.HNC]
  for field in list(config[utils.HNC]):
    if hasattr(spec.hierarchyController, field) and getattr(
        spec.hierarchyController, field) is not None:
      c[field] = getattr(spec.hierarchyController, field)



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Deploy a container to Cloud Run that will run to completion."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.run import config_changes
from googlecloudsdk.command_lib.run import connection_context
from googlecloudsdk.command_lib.run import flags
from googlecloudsdk.command_lib.run import messages_util
from googlecloudsdk.command_lib.run import pretty_print
from googlecloudsdk.command_lib.run import resource_args
from googlecloudsdk.command_lib.run import serverless_operations
from googlecloudsdk.command_lib.run import stages
from googlecloudsdk.command_lib.util.concepts import concept_parsers
from googlecloudsdk.command_lib.util.concepts import presentation_specs
from googlecloudsdk.core import log
from googlecloudsdk.core.console import progress_tracker


@base.ReleaseTracks(base.ReleaseTrack.BETA)
class Create(base.Command):
  """Create a Cloud Run job."""

  detailed_help = {
      'DESCRIPTION':
          """\
          Creates a new Cloud Run job.
          """,
      'EXAMPLES':
          """\
          To deploy a new job `my-data-transformation` on Cloud Run:

              $ {command} my-data-transformation --image=us-docker.pkg.dev/project/image

          You may also omit the job name. Then a prompt will be displayed
          with a suggested default value:

              $ {command} --image=us-docker.pkg.dev/project/image
          """,
  }

  @staticmethod
  def CommonArgs(parser):
    # Flags not specific to any platform
    job_presentation = presentation_specs.ResourcePresentationSpec(
        'JOB',
        resource_args.GetJobResourceSpec(prompt=True),
        'Job to create.',
        required=True,
        prefixes=False)
    flags.AddImageArg(
        parser, image='us-docker.pkg.dev/cloudrun/container/job:latest')
    flags.AddLabelsFlag(parser)
    flags.AddParallelismFlag(parser)
    flags.AddTasksFlag(parser)
    flags.AddMaxRetriesFlag(parser)
    flags.AddTaskTimeoutFlags(parser)
    flags.AddServiceAccountFlag(parser, managed_only=True)
    flags.AddMutexEnvVarsFlagsForCreate(parser)
    flags.AddSetCloudSQLFlag(parser)
    flags.AddVpcConnectorArg(parser)
    flags.AddEgressSettingsFlag(parser)
    flags.AddSetSecretsFlag(parser)
    flags.AddMemoryFlag(parser)
    flags.AddCpuFlag(parser, managed_only=True)
    flags.AddCommandFlag(parser)
    flags.AddArgsFlag(parser)
    flags.AddClientNameAndVersionFlags(parser)
    flags.AddBinAuthzPolicyFlags(parser, with_clear=False)
    flags.AddBinAuthzBreakglassFlag(parser)
    flags.AddCmekKeyFlag(parser, with_clear=False)
    flags.AddSandboxArg(parser, hidden=True)
    flags.AddGeneralAnnotationFlags(parser)

    polling_group = parser.add_mutually_exclusive_group()
    flags.AddAsyncFlag(polling_group)
    flags.AddWaitForCompletionFlag(polling_group, implies_execute_now=True)

    flags.AddExecuteNowFlag(parser)

    concept_parsers.ConceptParser([job_presentation]).AddToParser(parser)
    # No output by default, can be overridden by --format
    parser.display_info.AddFormat('none')

  @staticmethod
  def Args(parser):
    Create.CommonArgs(parser)

  def Run(self, args):
    """Deploy a Job to Cloud Run."""
    job_ref = args.CONCEPTS.job.Parse()
    flags.ValidateResource(job_ref)

    conn_context = connection_context.GetConnectionContext(
        args, flags.Product.RUN, self.ReleaseTrack())
    changes = flags.GetJobConfigurationChanges(args)
    changes.append(
        config_changes.SetLaunchStageAnnotationChange(self.ReleaseTrack()))

    execute_now = args.execute_now or args.wait
    execution = None

    with serverless_operations.Connect(conn_context) as operations:
      pretty_print.Info(
          messages_util.GetStartDeployMessage(conn_context, job_ref, 'Creating',
                                              'job'))
      if execute_now:
        header_msg = 'Creating and running job...'
      else:
        header_msg = 'Creating job...'
      with progress_tracker.StagedProgressTracker(
          header_msg,
          stages.JobStages(
              execute_now=execute_now, include_completion=args.wait),
          failure_message='Job failed to deploy',
          suppress_output=args.async_) as tracker:
        job = operations.CreateJob(
            job_ref, changes, tracker, asyn=(args.async_ and not execute_now))
        if execute_now:
          execution = operations.RunJob(job_ref, args.wait, tracker,
                                        args.async_, self.ReleaseTrack())

      if args.async_ and not execute_now:
        pretty_print.Success('Job [{{bold}}{job}{{reset}}] is being created '
                             'asynchronously.'.format(job=job.name))
      else:
        job = operations.GetJob(job_ref)
        operation = 'been created'
        if args.wait:
          operation += ' and completed execution [{}]'.format(execution.name)
        elif execute_now:
          operation += ' and started running execution [{}]'.format(
              execution.name)

        pretty_print.Success('Job [{{bold}}{job}{{reset}}] has successfully '
                             '{operation}.'.format(
                                 job=job.name, operation=operation))

      msg = ''
      if execute_now:
        msg += messages_util.GetExecutionCreatedMessage(self.ReleaseTrack(),
                                                        execution)
        msg += '\n'
      msg += messages_util.GetRunJobMessage(
          self.ReleaseTrack(), job.name, repeat=execute_now)
      log.status.Print(msg)
      return job


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class AlphaCreate(Create):
  """Deploy a new job to Cloud Run."""

  @staticmethod
  def Args(parser):
    Create.CommonArgs(parser)
    flags.AddVpcNetworkFlags(parser, resource_kind='Job')
    flags.AddVpcSubnetFlags(parser, resource_kind='Job')
    flags.AddVpcNetworkTagsFlags(parser, resource_kind='Job')
    flags.AddRuntimeFlag(parser)



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""List regions available to Google Cloud Functions."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.functions.v1 import util
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.functions import flags
from googlecloudsdk.command_lib.functions.v1.regions.list import command as command_v1
from googlecloudsdk.command_lib.functions.v2.regions.list import command as command_v2


def _CommonArgs(parser):
  parser.display_info.AddFormat('table(name)')
  parser.display_info.AddUriFunc(flags.GetLocationsUri)
  flags.AddGen2Flag(parser)


@base.ReleaseTracks(base.ReleaseTrack.GA)
class List(base.ListCommand):
  """List regions available to Google Cloud Functions."""

  @staticmethod
  def Args(parser):
    """Registers flags for this command."""
    _CommonArgs(parser)

  @util.CatchHTTPErrorRaiseHTTPException
  def Run(self, args):
    """This is what gets called when the user runs this command.

    Args:
      args: an argparse namespace. All the arguments that were provided to this
        command invocation.

    Returns:
      List of cloudfunctions_v1.Location or cloudfunctions_v2.Location:
        List of GCF regions

    Raises:
      FunctionsError: If the user doesn't confirm on prompt.
    """
    if flags.ShouldUseGen2():
      return command_v2.Run(args, self.ReleaseTrack())
    else:
      return command_v1.Run(args)


@base.ReleaseTracks(base.ReleaseTrack.BETA)
class ListBeta(List):
  """List regions available to Google Cloud Functions."""


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class ListAlpha(ListBeta):
  """List regions available to Google Cloud Functions."""



"""Generated client library for gkemulticloud version v1."""
# NOTE: This file is autogenerated and should not be edited by hand.

from __future__ import absolute_import

from apitools.base.py import base_api
from googlecloudsdk.generated_clients.apis.gkemulticloud.v1 import gkemulticloud_v1_messages as messages


class GkemulticloudV1(base_api.BaseApiClient):
  """Generated client library for service gkemulticloud version v1."""

  MESSAGES_MODULE = messages
  BASE_URL = 'https://gkemulticloud.googleapis.com/'
  MTLS_BASE_URL = 'https://gkemulticloud.mtls.googleapis.com/'

  _PACKAGE = 'gkemulticloud'
  _SCOPES = ['https://www.googleapis.com/auth/cloud-platform']
  _VERSION = 'v1'
  _CLIENT_ID = 'CLIENT_ID'
  _CLIENT_SECRET = 'CLIENT_SECRET'
  _USER_AGENT = 'google-cloud-sdk'
  _CLIENT_CLASS_NAME = 'GkemulticloudV1'
  _URL_VERSION = 'v1'
  _API_KEY = None

  def __init__(self, url='', credentials=None,
               get_credentials=True, http=None, model=None,
               log_request=False, log_response=False,
               credentials_args=None, default_global_params=None,
               additional_http_headers=None, response_encoding=None):
    """Create a new gkemulticloud handle."""
    url = url or self.BASE_URL
    super(GkemulticloudV1, self).__init__(
        url, credentials=credentials,
        get_credentials=get_credentials, http=http, model=model,
        log_request=log_request, log_response=log_response,
        credentials_args=credentials_args,
        default_global_params=default_global_params,
        additional_http_headers=additional_http_headers,
        response_encoding=response_encoding)
    self.projects_locations_attachedClusters = self.ProjectsLocationsAttachedClustersService(self)
    self.projects_locations_awsClusters_awsNodePools = self.ProjectsLocationsAwsClustersAwsNodePoolsService(self)
    self.projects_locations_awsClusters_well_known = self.ProjectsLocationsAwsClustersWellKnownService(self)
    self.projects_locations_awsClusters = self.ProjectsLocationsAwsClustersService(self)
    self.projects_locations_azureClients = self.ProjectsLocationsAzureClientsService(self)
    self.projects_locations_azureClusters_azureNodePools = self.ProjectsLocationsAzureClustersAzureNodePoolsService(self)
    self.projects_locations_azureClusters_well_known = self.ProjectsLocationsAzureClustersWellKnownService(self)
    self.projects_locations_azureClusters = self.ProjectsLocationsAzureClustersService(self)
    self.projects_locations_operations = self.ProjectsLocationsOperationsService(self)
    self.projects_locations = self.ProjectsLocationsService(self)
    self.projects = self.ProjectsService(self)

  class ProjectsLocationsAttachedClustersService(base_api.BaseApiService):
    """Service class for the projects_locations_attachedClusters resource."""

    _NAME = 'projects_locations_attachedClusters'

    def __init__(self, client):
      super(GkemulticloudV1.ProjectsLocationsAttachedClustersService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a new AttachedCluster resource on a given Google Cloud Platform project and region. If successful, the response contains a newly created Operation resource that can be described to track the status of the operation.

      Args:
        request: (GkemulticloudProjectsLocationsAttachedClustersCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleLongrunningOperation) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/attachedClusters',
        http_method='POST',
        method_id='gkemulticloud.projects.locations.attachedClusters.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['attachedClusterId', 'validateOnly'],
        relative_path='v1/{+parent}/attachedClusters',
        request_field='googleCloudGkemulticloudV1AttachedCluster',
        request_type_name='GkemulticloudProjectsLocationsAttachedClustersCreateRequest',
        response_type_name='GoogleLongrunningOperation',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a specific AttachedCluster resource. If successful, the response contains a newly created Operation resource that can be described to track the status of the operation.

      Args:
        request: (GkemulticloudProjectsLocationsAttachedClustersDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleLongrunningOperation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/attachedClusters/{attachedClustersId}',
        http_method='DELETE',
        method_id='gkemulticloud.projects.locations.attachedClusters.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['allowMissing', 'etag', 'ignoreErrors', 'validateOnly'],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='GkemulticloudProjectsLocationsAttachedClustersDeleteRequest',
        response_type_name='GoogleLongrunningOperation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Describes a specific AttachedCluster resource.

      Args:
        request: (GkemulticloudProjectsLocationsAttachedClustersGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudGkemulticloudV1AttachedCluster) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/attachedClusters/{attachedClustersId}',
        http_method='GET',
        method_id='gkemulticloud.projects.locations.attachedClusters.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='GkemulticloudProjectsLocationsAttachedClustersGetRequest',
        response_type_name='GoogleCloudGkemulticloudV1AttachedCluster',
        supports_download=False,
    )

    def Import(self, request, global_params=None):
      r"""Imports creates a new AttachedCluster resource by importing an existing Fleet Membership resource. Attached Clusters created before the introduction of the Anthos Multi-Cloud API can be imported through this method. If successful, the response contains a newly created Operation resource that can be described to track the status of the operation.

      Args:
        request: (GkemulticloudProjectsLocationsAttachedClustersImportRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleLongrunningOperation) The response message.
      """
      config = self.GetMethodConfig('Import')
      return self._RunMethod(
          config, request, global_params=global_params)

    Import.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/attachedClusters:import',
        http_method='POST',
        method_id='gkemulticloud.projects.locations.attachedClusters.import',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=[],
        relative_path='v1/{+parent}/attachedClusters:import',
        request_field='googleCloudGkemulticloudV1ImportAttachedClusterRequest',
        request_type_name='GkemulticloudProjectsLocationsAttachedClustersImportRequest',
        response_type_name='GoogleLongrunningOperation',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists all AttachedCluster resources on a given Google Cloud project and region.

      Args:
        request: (GkemulticloudProjectsLocationsAttachedClustersListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudGkemulticloudV1ListAttachedClustersResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/attachedClusters',
        http_method='GET',
        method_id='gkemulticloud.projects.locations.attachedClusters.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['pageSize', 'pageToken'],
        relative_path='v1/{+parent}/attachedClusters',
        request_field='',
        request_type_name='GkemulticloudProjectsLocationsAttachedClustersListRequest',
        response_type_name='GoogleCloudGkemulticloudV1ListAttachedClustersResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates an AttachedCluster.

      Args:
        request: (GkemulticloudProjectsLocationsAttachedClustersPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleLongrunningOperation) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/attachedClusters/{attachedClustersId}',
        http_method='PATCH',
        method_id='gkemulticloud.projects.locations.attachedClusters.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask', 'validateOnly'],
        relative_path='v1/{+name}',
        request_field='googleCloudGkemulticloudV1AttachedCluster',
        request_type_name='GkemulticloudProjectsLocationsAttachedClustersPatchRequest',
        response_type_name='GoogleLongrunningOperation',
        supports_download=False,
    )

  class ProjectsLocationsAwsClustersAwsNodePoolsService(base_api.BaseApiService):
    """Service class for the projects_locations_awsClusters_awsNodePools resource."""

    _NAME = 'projects_locations_awsClusters_awsNodePools'

    def __init__(self, client):
      super(GkemulticloudV1.ProjectsLocationsAwsClustersAwsNodePoolsService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a new AwsNodePool, attached to a given AwsCluster. If successful, the response contains a newly created Operation resource that can be described to track the status of the operation.

      Args:
        request: (GkemulticloudProjectsLocationsAwsClustersAwsNodePoolsCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleLongrunningOperation) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/awsClusters/{awsClustersId}/awsNodePools',
        http_method='POST',
        method_id='gkemulticloud.projects.locations.awsClusters.awsNodePools.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['awsNodePoolId', 'validateOnly'],
        relative_path='v1/{+parent}/awsNodePools',
        request_field='googleCloudGkemulticloudV1AwsNodePool',
        request_type_name='GkemulticloudProjectsLocationsAwsClustersAwsNodePoolsCreateRequest',
        response_type_name='GoogleLongrunningOperation',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a specific AwsNodePool resource. If successful, the response contains a newly created Operation resource that can be described to track the status of the operation.

      Args:
        request: (GkemulticloudProjectsLocationsAwsClustersAwsNodePoolsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleLongrunningOperation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/awsClusters/{awsClustersId}/awsNodePools/{awsNodePoolsId}',
        http_method='DELETE',
        method_id='gkemulticloud.projects.locations.awsClusters.awsNodePools.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['allowMissing', 'etag', 'validateOnly'],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='GkemulticloudProjectsLocationsAwsClustersAwsNodePoolsDeleteRequest',
        response_type_name='GoogleLongrunningOperation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Describes a specific AwsNodePool resource.

      Args:
        request: (GkemulticloudProjectsLocationsAwsClustersAwsNodePoolsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudGkemulticloudV1AwsNodePool) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/awsClusters/{awsClustersId}/awsNodePools/{awsNodePoolsId}',
        http_method='GET',
        method_id='gkemulticloud.projects.locations.awsClusters.awsNodePools.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='GkemulticloudProjectsLocationsAwsClustersAwsNodePoolsGetRequest',
        response_type_name='GoogleCloudGkemulticloudV1AwsNodePool',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists all AwsNodePool resources on a given AwsCluster.

      Args:
        request: (GkemulticloudProjectsLocationsAwsClustersAwsNodePoolsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudGkemulticloudV1ListAwsNodePoolsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/awsClusters/{awsClustersId}/awsNodePools',
        http_method='GET',
        method_id='gkemulticloud.projects.locations.awsClusters.awsNodePools.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['pageSize', 'pageToken'],
        relative_path='v1/{+parent}/awsNodePools',
        request_field='',
        request_type_name='GkemulticloudProjectsLocationsAwsClustersAwsNodePoolsListRequest',
        response_type_name='GoogleCloudGkemulticloudV1ListAwsNodePoolsResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates an AwsNodePool.

      Args:
        request: (GkemulticloudProjectsLocationsAwsClustersAwsNodePoolsPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleLongrunningOperation) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/awsClusters/{awsClustersId}/awsNodePools/{awsNodePoolsId}',
        http_method='PATCH',
        method_id='gkemulticloud.projects.locations.awsClusters.awsNodePools.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask', 'validateOnly'],
        relative_path='v1/{+name}',
        request_field='googleCloudGkemulticloudV1AwsNodePool',
        request_type_name='GkemulticloudProjectsLocationsAwsClustersAwsNodePoolsPatchRequest',
        response_type_name='GoogleLongrunningOperation',
        supports_download=False,
    )

  class ProjectsLocationsAwsClustersWellKnownService(base_api.BaseApiService):
    """Service class for the projects_locations_awsClusters_well_known resource."""

    _NAME = 'projects_locations_awsClusters_well_known'

    def __init__(self, client):
      super(GkemulticloudV1.ProjectsLocationsAwsClustersWellKnownService, self).__init__(client)
      self._upload_configs = {
          }

    def GetOpenid_configuration(self, request, global_params=None):
      r"""Gets the OIDC discovery document for the cluster. See the [OpenID Connect Discovery 1.0 specification](https://openid.net/specs/openid-connect-discovery-1_0.html) for details.

      Args:
        request: (GkemulticloudProjectsLocationsAwsClustersWellKnownGetOpenidConfigurationRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudGkemulticloudV1AwsOpenIdConfig) The response message.
      """
      config = self.GetMethodConfig('GetOpenid_configuration')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetOpenid_configuration.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/awsClusters/{awsClustersId}/.well-known/openid-configuration',
        http_method='GET',
        method_id='gkemulticloud.projects.locations.awsClusters.well-known.getOpenid-configuration',
        ordered_params=['awsCluster'],
        path_params=['awsCluster'],
        query_params=[],
        relative_path='v1/{+awsCluster}/.well-known/openid-configuration',
        request_field='',
        request_type_name='GkemulticloudProjectsLocationsAwsClustersWellKnownGetOpenidConfigurationRequest',
        response_type_name='GoogleCloudGkemulticloudV1AwsOpenIdConfig',
        supports_download=False,
    )

  class ProjectsLocationsAwsClustersService(base_api.BaseApiService):
    """Service class for the projects_locations_awsClusters resource."""

    _NAME = 'projects_locations_awsClusters'

    def __init__(self, client):
      super(GkemulticloudV1.ProjectsLocationsAwsClustersService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a new AwsCluster resource on a given Google Cloud Platform project and region. If successful, the response contains a newly created Operation resource that can be described to track the status of the operation.

      Args:
        request: (GkemulticloudProjectsLocationsAwsClustersCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleLongrunningOperation) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/awsClusters',
        http_method='POST',
        method_id='gkemulticloud.projects.locations.awsClusters.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['awsClusterId', 'validateOnly'],
        relative_path='v1/{+parent}/awsClusters',
        request_field='googleCloudGkemulticloudV1AwsCluster',
        request_type_name='GkemulticloudProjectsLocationsAwsClustersCreateRequest',
        response_type_name='GoogleLongrunningOperation',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a specific AwsCluster resource. Fails if the cluster has one or more associated AwsNodePool resources. If successful, the response contains a newly created Operation resource that can be described to track the status of the operation.

      Args:
        request: (GkemulticloudProjectsLocationsAwsClustersDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleLongrunningOperation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/awsClusters/{awsClustersId}',
        http_method='DELETE',
        method_id='gkemulticloud.projects.locations.awsClusters.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['allowMissing', 'etag', 'validateOnly'],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='GkemulticloudProjectsLocationsAwsClustersDeleteRequest',
        response_type_name='GoogleLongrunningOperation',
        supports_download=False,
    )

    def GenerateAwsAccessToken(self, request, global_params=None):
      r"""Generates a short-lived access token to authenticate to a given AwsCluster resource.

      Args:
        request: (GkemulticloudProjectsLocationsAwsClustersGenerateAwsAccessTokenRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudGkemulticloudV1GenerateAwsAccessTokenResponse) The response message.
      """
      config = self.GetMethodConfig('GenerateAwsAccessToken')
      return self._RunMethod(
          config, request, global_params=global_params)

    GenerateAwsAccessToken.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/awsClusters/{awsClustersId}:generateAwsAccessToken',
        http_method='GET',
        method_id='gkemulticloud.projects.locations.awsClusters.generateAwsAccessToken',
        ordered_params=['awsCluster'],
        path_params=['awsCluster'],
        query_params=[],
        relative_path='v1/{+awsCluster}:generateAwsAccessToken',
        request_field='',
        request_type_name='GkemulticloudProjectsLocationsAwsClustersGenerateAwsAccessTokenRequest',
        response_type_name='GoogleCloudGkemulticloudV1GenerateAwsAccessTokenResponse',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Describes a specific AwsCluster resource.

      Args:
        request: (GkemulticloudProjectsLocationsAwsClustersGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudGkemulticloudV1AwsCluster) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/awsClusters/{awsClustersId}',
        http_method='GET',
        method_id='gkemulticloud.projects.locations.awsClusters.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='GkemulticloudProjectsLocationsAwsClustersGetRequest',
        response_type_name='GoogleCloudGkemulticloudV1AwsCluster',
        supports_download=False,
    )

    def GetJwks(self, request, global_params=None):
      r"""Gets the public component of the cluster signing keys in JSON Web Key format.

      Args:
        request: (GkemulticloudProjectsLocationsAwsClustersGetJwksRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudGkemulticloudV1AwsJsonWebKeys) The response message.
      """
      config = self.GetMethodConfig('GetJwks')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetJwks.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/awsClusters/{awsClustersId}/jwks',
        http_method='GET',
        method_id='gkemulticloud.projects.locations.awsClusters.getJwks',
        ordered_params=['awsCluster'],
        path_params=['awsCluster'],
        query_params=[],
        relative_path='v1/{+awsCluster}/jwks',
        request_field='',
        request_type_name='GkemulticloudProjectsLocationsAwsClustersGetJwksRequest',
        response_type_name='GoogleCloudGkemulticloudV1AwsJsonWebKeys',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists all AwsCluster resources on a given Google Cloud project and region.

      Args:
        request: (GkemulticloudProjectsLocationsAwsClustersListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudGkemulticloudV1ListAwsClustersResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/awsClusters',
        http_method='GET',
        method_id='gkemulticloud.projects.locations.awsClusters.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['pageSize', 'pageToken'],
        relative_path='v1/{+parent}/awsClusters',
        request_field='',
        request_type_name='GkemulticloudProjectsLocationsAwsClustersListRequest',
        response_type_name='GoogleCloudGkemulticloudV1ListAwsClustersResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates an AwsCluster.

      Args:
        request: (GkemulticloudProjectsLocationsAwsClustersPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleLongrunningOperation) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/awsClusters/{awsClustersId}',
        http_method='PATCH',
        method_id='gkemulticloud.projects.locations.awsClusters.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask', 'validateOnly'],
        relative_path='v1/{+name}',
        request_field='googleCloudGkemulticloudV1AwsCluster',
        request_type_name='GkemulticloudProjectsLocationsAwsClustersPatchRequest',
        response_type_name='GoogleLongrunningOperation',
        supports_download=False,
    )

  class ProjectsLocationsAzureClientsService(base_api.BaseApiService):
    """Service class for the projects_locations_azureClients resource."""

    _NAME = 'projects_locations_azureClients'

    def __init__(self, client):
      super(GkemulticloudV1.ProjectsLocationsAzureClientsService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a new AzureClient resource on a given Google Cloud project and region. `AzureClient` resources hold client authentication information needed by the Anthos Multicloud API to manage Azure resources on your Azure subscription on your behalf. If successful, the response contains a newly created Operation resource that can be described to track the status of the operation.

      Args:
        request: (GkemulticloudProjectsLocationsAzureClientsCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleLongrunningOperation) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/azureClients',
        http_method='POST',
        method_id='gkemulticloud.projects.locations.azureClients.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['azureClientId', 'validateOnly'],
        relative_path='v1/{+parent}/azureClients',
        request_field='googleCloudGkemulticloudV1AzureClient',
        request_type_name='GkemulticloudProjectsLocationsAzureClientsCreateRequest',
        response_type_name='GoogleLongrunningOperation',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a specific AzureClient resource. If the client is used by one or more clusters, deletion will fail and a `FAILED_PRECONDITION` error will be returned. If successful, the response contains a newly created Operation resource that can be described to track the status of the operation.

      Args:
        request: (GkemulticloudProjectsLocationsAzureClientsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleLongrunningOperation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/azureClients/{azureClientsId}',
        http_method='DELETE',
        method_id='gkemulticloud.projects.locations.azureClients.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['allowMissing', 'validateOnly'],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='GkemulticloudProjectsLocationsAzureClientsDeleteRequest',
        response_type_name='GoogleLongrunningOperation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Describes a specific AzureClient resource.

      Args:
        request: (GkemulticloudProjectsLocationsAzureClientsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudGkemulticloudV1AzureClient) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/azureClients/{azureClientsId}',
        http_method='GET',
        method_id='gkemulticloud.projects.locations.azureClients.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='GkemulticloudProjectsLocationsAzureClientsGetRequest',
        response_type_name='GoogleCloudGkemulticloudV1AzureClient',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists all AzureClient resources on a given Google Cloud project and region.

      Args:
        request: (GkemulticloudProjectsLocationsAzureClientsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudGkemulticloudV1ListAzureClientsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/azureClients',
        http_method='GET',
        method_id='gkemulticloud.projects.locations.azureClients.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['pageSize', 'pageToken'],
        relative_path='v1/{+parent}/azureClients',
        request_field='',
        request_type_name='GkemulticloudProjectsLocationsAzureClientsListRequest',
        response_type_name='GoogleCloudGkemulticloudV1ListAzureClientsResponse',
        supports_download=False,
    )

  class ProjectsLocationsAzureClustersAzureNodePoolsService(base_api.BaseApiService):
    """Service class for the projects_locations_azureClusters_azureNodePools resource."""

    _NAME = 'projects_locations_azureClusters_azureNodePools'

    def __init__(self, client):
      super(GkemulticloudV1.ProjectsLocationsAzureClustersAzureNodePoolsService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a new AzureNodePool, attached to a given AzureCluster. If successful, the response contains a newly created Operation resource that can be described to track the status of the operation.

      Args:
        request: (GkemulticloudProjectsLocationsAzureClustersAzureNodePoolsCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleLongrunningOperation) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/azureClusters/{azureClustersId}/azureNodePools',
        http_method='POST',
        method_id='gkemulticloud.projects.locations.azureClusters.azureNodePools.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['azureNodePoolId', 'validateOnly'],
        relative_path='v1/{+parent}/azureNodePools',
        request_field='googleCloudGkemulticloudV1AzureNodePool',
        request_type_name='GkemulticloudProjectsLocationsAzureClustersAzureNodePoolsCreateRequest',
        response_type_name='GoogleLongrunningOperation',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a specific AzureNodePool resource. If successful, the response contains a newly created Operation resource that can be described to track the status of the operation.

      Args:
        request: (GkemulticloudProjectsLocationsAzureClustersAzureNodePoolsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleLongrunningOperation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/azureClusters/{azureClustersId}/azureNodePools/{azureNodePoolsId}',
        http_method='DELETE',
        method_id='gkemulticloud.projects.locations.azureClusters.azureNodePools.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['allowMissing', 'etag', 'validateOnly'],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='GkemulticloudProjectsLocationsAzureClustersAzureNodePoolsDeleteRequest',
        response_type_name='GoogleLongrunningOperation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Describes a specific AzureNodePool resource.

      Args:
        request: (GkemulticloudProjectsLocationsAzureClustersAzureNodePoolsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudGkemulticloudV1AzureNodePool) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/azureClusters/{azureClustersId}/azureNodePools/{azureNodePoolsId}',
        http_method='GET',
        method_id='gkemulticloud.projects.locations.azureClusters.azureNodePools.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='GkemulticloudProjectsLocationsAzureClustersAzureNodePoolsGetRequest',
        response_type_name='GoogleCloudGkemulticloudV1AzureNodePool',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists all AzureNodePool resources on a given AzureCluster.

      Args:
        request: (GkemulticloudProjectsLocationsAzureClustersAzureNodePoolsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudGkemulticloudV1ListAzureNodePoolsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/azureClusters/{azureClustersId}/azureNodePools',
        http_method='GET',
        method_id='gkemulticloud.projects.locations.azureClusters.azureNodePools.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['pageSize', 'pageToken'],
        relative_path='v1/{+parent}/azureNodePools',
        request_field='',
        request_type_name='GkemulticloudProjectsLocationsAzureClustersAzureNodePoolsListRequest',
        response_type_name='GoogleCloudGkemulticloudV1ListAzureNodePoolsResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates an AzureNodePool.

      Args:
        request: (GkemulticloudProjectsLocationsAzureClustersAzureNodePoolsPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleLongrunningOperation) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/azureClusters/{azureClustersId}/azureNodePools/{azureNodePoolsId}',
        http_method='PATCH',
        method_id='gkemulticloud.projects.locations.azureClusters.azureNodePools.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask', 'validateOnly'],
        relative_path='v1/{+name}',
        request_field='googleCloudGkemulticloudV1AzureNodePool',
        request_type_name='GkemulticloudProjectsLocationsAzureClustersAzureNodePoolsPatchRequest',
        response_type_name='GoogleLongrunningOperation',
        supports_download=False,
    )

  class ProjectsLocationsAzureClustersWellKnownService(base_api.BaseApiService):
    """Service class for the projects_locations_azureClusters_well_known resource."""

    _NAME = 'projects_locations_azureClusters_well_known'

    def __init__(self, client):
      super(GkemulticloudV1.ProjectsLocationsAzureClustersWellKnownService, self).__init__(client)
      self._upload_configs = {
          }

    def GetOpenid_configuration(self, request, global_params=None):
      r"""Gets the OIDC discovery document for the cluster. See the [OpenID Connect Discovery 1.0 specification](https://openid.net/specs/openid-connect-discovery-1_0.html) for details.

      Args:
        request: (GkemulticloudProjectsLocationsAzureClustersWellKnownGetOpenidConfigurationRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudGkemulticloudV1AzureOpenIdConfig) The response message.
      """
      config = self.GetMethodConfig('GetOpenid_configuration')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetOpenid_configuration.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/azureClusters/{azureClustersId}/.well-known/openid-configuration',
        http_method='GET',
        method_id='gkemulticloud.projects.locations.azureClusters.well-known.getOpenid-configuration',
        ordered_params=['azureCluster'],
        path_params=['azureCluster'],
        query_params=[],
        relative_path='v1/{+azureCluster}/.well-known/openid-configuration',
        request_field='',
        request_type_name='GkemulticloudProjectsLocationsAzureClustersWellKnownGetOpenidConfigurationRequest',
        response_type_name='GoogleCloudGkemulticloudV1AzureOpenIdConfig',
        supports_download=False,
    )

  class ProjectsLocationsAzureClustersService(base_api.BaseApiService):
    """Service class for the projects_locations_azureClusters resource."""

    _NAME = 'projects_locations_azureClusters'

    def __init__(self, client):
      super(GkemulticloudV1.ProjectsLocationsAzureClustersService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a new AzureCluster resource on a given Google Cloud Platform project and region. If successful, the response contains a newly created Operation resource that can be described to track the status of the operation.

      Args:
        request: (GkemulticloudProjectsLocationsAzureClustersCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleLongrunningOperation) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/azureClusters',
        http_method='POST',
        method_id='gkemulticloud.projects.locations.azureClusters.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['azureClusterId', 'validateOnly'],
        relative_path='v1/{+parent}/azureClusters',
        request_field='googleCloudGkemulticloudV1AzureCluster',
        request_type_name='GkemulticloudProjectsLocationsAzureClustersCreateRequest',
        response_type_name='GoogleLongrunningOperation',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a specific AzureCluster resource. Fails if the cluster has one or more associated AzureNodePool resources. If successful, the response contains a newly created Operation resource that can be described to track the status of the operation.

      Args:
        request: (GkemulticloudProjectsLocationsAzureClustersDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleLongrunningOperation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/azureClusters/{azureClustersId}',
        http_method='DELETE',
        method_id='gkemulticloud.projects.locations.azureClusters.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['allowMissing', 'etag', 'validateOnly'],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='GkemulticloudProjectsLocationsAzureClustersDeleteRequest',
        response_type_name='GoogleLongrunningOperation',
        supports_download=False,
    )

    def GenerateAzureAccessToken(self, request, global_params=None):
      r"""Generates a short-lived access token to authenticate to a given AzureCluster resource.

      Args:
        request: (GkemulticloudProjectsLocationsAzureClustersGenerateAzureAccessTokenRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudGkemulticloudV1GenerateAzureAccessTokenResponse) The response message.
      """
      config = self.GetMethodConfig('GenerateAzureAccessToken')
      return self._RunMethod(
          config, request, global_params=global_params)

    GenerateAzureAccessToken.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/azureClusters/{azureClustersId}:generateAzureAccessToken',
        http_method='GET',
        method_id='gkemulticloud.projects.locations.azureClusters.generateAzureAccessToken',
        ordered_params=['azureCluster'],
        path_params=['azureCluster'],
        query_params=[],
        relative_path='v1/{+azureCluster}:generateAzureAccessToken',
        request_field='',
        request_type_name='GkemulticloudProjectsLocationsAzureClustersGenerateAzureAccessTokenRequest',
        response_type_name='GoogleCloudGkemulticloudV1GenerateAzureAccessTokenResponse',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Describes a specific AzureCluster resource.

      Args:
        request: (GkemulticloudProjectsLocationsAzureClustersGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudGkemulticloudV1AzureCluster) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/azureClusters/{azureClustersId}',
        http_method='GET',
        method_id='gkemulticloud.projects.locations.azureClusters.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='GkemulticloudProjectsLocationsAzureClustersGetRequest',
        response_type_name='GoogleCloudGkemulticloudV1AzureCluster',
        supports_download=False,
    )

    def GetJwks(self, request, global_params=None):
      r"""Gets the public component of the cluster signing keys in JSON Web Key format.

      Args:
        request: (GkemulticloudProjectsLocationsAzureClustersGetJwksRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudGkemulticloudV1AzureJsonWebKeys) The response message.
      """
      config = self.GetMethodConfig('GetJwks')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetJwks.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/azureClusters/{azureClustersId}/jwks',
        http_method='GET',
        method_id='gkemulticloud.projects.locations.azureClusters.getJwks',
        ordered_params=['azureCluster'],
        path_params=['azureCluster'],
        query_params=[],
        relative_path='v1/{+azureCluster}/jwks',
        request_field='',
        request_type_name='GkemulticloudProjectsLocationsAzureClustersGetJwksRequest',
        response_type_name='GoogleCloudGkemulticloudV1AzureJsonWebKeys',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists all AzureCluster resources on a given Google Cloud project and region.

      Args:
        request: (GkemulticloudProjectsLocationsAzureClustersListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudGkemulticloudV1ListAzureClustersResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/azureClusters',
        http_method='GET',
        method_id='gkemulticloud.projects.locations.azureClusters.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['pageSize', 'pageToken'],
        relative_path='v1/{+parent}/azureClusters',
        request_field='',
        request_type_name='GkemulticloudProjectsLocationsAzureClustersListRequest',
        response_type_name='GoogleCloudGkemulticloudV1ListAzureClustersResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates an AzureCluster.

      Args:
        request: (GkemulticloudProjectsLocationsAzureClustersPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleLongrunningOperation) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/azureClusters/{azureClustersId}',
        http_method='PATCH',
        method_id='gkemulticloud.projects.locations.azureClusters.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask', 'validateOnly'],
        relative_path='v1/{+name}',
        request_field='googleCloudGkemulticloudV1AzureCluster',
        request_type_name='GkemulticloudProjectsLocationsAzureClustersPatchRequest',
        response_type_name='GoogleLongrunningOperation',
        supports_download=False,
    )

  class ProjectsLocationsOperationsService(base_api.BaseApiService):
    """Service class for the projects_locations_operations resource."""

    _NAME = 'projects_locations_operations'

    def __init__(self, client):
      super(GkemulticloudV1.ProjectsLocationsOperationsService, self).__init__(client)
      self._upload_configs = {
          }

    def Cancel(self, request, global_params=None):
      r"""Starts asynchronous cancellation on a long-running operation. The server makes a best effort to cancel the operation, but success is not guaranteed. If the server doesn't support this method, it returns `google.rpc.Code.UNIMPLEMENTED`. Clients can use Operations.GetOperation or other methods to check whether the cancellation succeeded or whether the operation completed despite cancellation. On successful cancellation, the operation is not deleted; instead, it becomes an operation with an Operation.error value with a google.rpc.Status.code of 1, corresponding to `Code.CANCELLED`.

      Args:
        request: (GkemulticloudProjectsLocationsOperationsCancelRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleProtobufEmpty) The response message.
      """
      config = self.GetMethodConfig('Cancel')
      return self._RunMethod(
          config, request, global_params=global_params)

    Cancel.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/operations/{operationsId}:cancel',
        http_method='POST',
        method_id='gkemulticloud.projects.locations.operations.cancel',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}:cancel',
        request_field='googleLongrunningCancelOperationRequest',
        request_type_name='GkemulticloudProjectsLocationsOperationsCancelRequest',
        response_type_name='GoogleProtobufEmpty',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a long-running operation. This method indicates that the client is no longer interested in the operation result. It does not cancel the operation. If the server doesn't support this method, it returns `google.rpc.Code.UNIMPLEMENTED`.

      Args:
        request: (GkemulticloudProjectsLocationsOperationsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleProtobufEmpty) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/operations/{operationsId}',
        http_method='DELETE',
        method_id='gkemulticloud.projects.locations.operations.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='GkemulticloudProjectsLocationsOperationsDeleteRequest',
        response_type_name='GoogleProtobufEmpty',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets the latest state of a long-running operation. Clients can use this method to poll the operation result at intervals as recommended by the API service.

      Args:
        request: (GkemulticloudProjectsLocationsOperationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleLongrunningOperation) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/operations/{operationsId}',
        http_method='GET',
        method_id='gkemulticloud.projects.locations.operations.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='GkemulticloudProjectsLocationsOperationsGetRequest',
        response_type_name='GoogleLongrunningOperation',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists operations that match the specified filter in the request. If the server doesn't support this method, it returns `UNIMPLEMENTED`. NOTE: the `name` binding allows API services to override the binding to use different resource name schemes, such as `users/*/operations`. To override the binding, API services can add a binding such as `"/v1/{name=users/*}/operations"` to their service configuration. For backwards compatibility, the default name includes the operations collection id, however overriding users must ensure the name binding is the parent resource, without the operations collection id.

      Args:
        request: (GkemulticloudProjectsLocationsOperationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleLongrunningListOperationsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/operations',
        http_method='GET',
        method_id='gkemulticloud.projects.locations.operations.list',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['filter', 'pageSize', 'pageToken'],
        relative_path='v1/{+name}/operations',
        request_field='',
        request_type_name='GkemulticloudProjectsLocationsOperationsListRequest',
        response_type_name='GoogleLongrunningListOperationsResponse',
        supports_download=False,
    )

  class ProjectsLocationsService(base_api.BaseApiService):
    """Service class for the projects_locations resource."""

    _NAME = 'projects_locations'

    def __init__(self, client):
      super(GkemulticloudV1.ProjectsLocationsService, self).__init__(client)
      self._upload_configs = {
          }

    def GenerateAttachedClusterInstallManifest(self, request, global_params=None):
      r"""Generates the install manifest to be installed on the target cluster.

      Args:
        request: (GkemulticloudProjectsLocationsGenerateAttachedClusterInstallManifestRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudGkemulticloudV1GenerateAttachedClusterInstallManifestResponse) The response message.
      """
      config = self.GetMethodConfig('GenerateAttachedClusterInstallManifest')
      return self._RunMethod(
          config, request, global_params=global_params)

    GenerateAttachedClusterInstallManifest.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}:generateAttachedClusterInstallManifest',
        http_method='GET',
        method_id='gkemulticloud.projects.locations.generateAttachedClusterInstallManifest',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['attachedClusterId', 'platformVersion'],
        relative_path='v1/{+parent}:generateAttachedClusterInstallManifest',
        request_field='',
        request_type_name='GkemulticloudProjectsLocationsGenerateAttachedClusterInstallManifestRequest',
        response_type_name='GoogleCloudGkemulticloudV1GenerateAttachedClusterInstallManifestResponse',
        supports_download=False,
    )

    def GetAttachedServerConfig(self, request, global_params=None):
      r"""Returns information, such as supported Kubernetes versions, on a given Google Cloud location.

      Args:
        request: (GkemulticloudProjectsLocationsGetAttachedServerConfigRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudGkemulticloudV1AttachedServerConfig) The response message.
      """
      config = self.GetMethodConfig('GetAttachedServerConfig')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetAttachedServerConfig.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/attachedServerConfig',
        http_method='GET',
        method_id='gkemulticloud.projects.locations.getAttachedServerConfig',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='GkemulticloudProjectsLocationsGetAttachedServerConfigRequest',
        response_type_name='GoogleCloudGkemulticloudV1AttachedServerConfig',
        supports_download=False,
    )

    def GetAwsServerConfig(self, request, global_params=None):
      r"""Returns information, such as supported AWS regions and Kubernetes versions, on a given Google Cloud location.

      Args:
        request: (GkemulticloudProjectsLocationsGetAwsServerConfigRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudGkemulticloudV1AwsServerConfig) The response message.
      """
      config = self.GetMethodConfig('GetAwsServerConfig')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetAwsServerConfig.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/awsServerConfig',
        http_method='GET',
        method_id='gkemulticloud.projects.locations.getAwsServerConfig',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='GkemulticloudProjectsLocationsGetAwsServerConfigRequest',
        response_type_name='GoogleCloudGkemulticloudV1AwsServerConfig',
        supports_download=False,
    )

    def GetAzureServerConfig(self, request, global_params=None):
      r"""Returns information, such as supported Azure regions and Kubernetes versions, on a given Google Cloud location.

      Args:
        request: (GkemulticloudProjectsLocationsGetAzureServerConfigRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudGkemulticloudV1AzureServerConfig) The response message.
      """
      config = self.GetMethodConfig('GetAzureServerConfig')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetAzureServerConfig.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/azureServerConfig',
        http_method='GET',
        method_id='gkemulticloud.projects.locations.getAzureServerConfig',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='GkemulticloudProjectsLocationsGetAzureServerConfigRequest',
        response_type_name='GoogleCloudGkemulticloudV1AzureServerConfig',
        supports_download=False,
    )

  class ProjectsService(base_api.BaseApiService):
    """Service class for the projects resource."""

    _NAME = 'projects'

    def __init__(self, client):
      super(GkemulticloudV1.ProjectsService, self).__init__(client)
      self._upload_configs = {
          }



# -*- coding: utf-8 -*- #
# Copyright 2022 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""`gcloud dataplex data-taxonomies atttributes update` command."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.dataplex import data_taxonomy
from googlecloudsdk.api_lib.dataplex import util as dataplex_util
from googlecloudsdk.api_lib.util import exceptions as gcloud_exception
from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions

from googlecloudsdk.command_lib.dataplex import resource_args
from googlecloudsdk.command_lib.util.args import labels_util
from googlecloudsdk.core import log


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class Update(base.Command):
  """Update a Dataplex Data Attribute."""

  detailed_help = {
      'EXAMPLES':
          """\

          To update Data Attribute `test-attribute` for Data Taxonomy `test-datataxonomy` in project `test-dataplex`
          at location `us-central1` with description as `test description` ,  display name as `displayName`,
          resource-readers as user:test@google.com, resource-writers as user:test@google.com, resource-owner as user:test@google.com run:
          data-readers as user:test@google.com and parent as `test-attribute-parent`, run:

              $ {command} test-attribute --location=us-central1 --project=test-dataplex'
              --data_taxonomy=test-datataxonomy --description='test description'
              --display-name='displayName' --resource-readers='user:test@google.com'
              --resource-writers='user:test@google.com' --resource-owners='user:test@google.com'
              --data-readers='user:test@google.com'
              --parent='test-attribute-parent'

          """,
  }

  @staticmethod
  def Args(parser):
    resource_args.AddAttributeResourceArg(parser, 'to update.')
    parser.add_argument(
        '--description',
        required=False,
        help='Description of the Data Attribute.')
    parser.add_argument(
        '--display-name',
        required=False,
        help='Display Name of the Data Attribute.')
    parser.add_argument(
        '--etag',
        required=False,
        help='etag value for particular Data Attribute resource.')
    parser.add_argument(
        '--parent',
        required=False,
        help='Parent Data Attribute for the defined Data Attribute. It can be attribute name or fully qualified attribute name.')
    resource_acces_sepc = parser.add_group(
        required=False,
        help='Spec related to Dataplex Resource.Specified when applied to a resource (eg: Google Cloud Storage bucket, BigQuery, dataset, BigQuery table).'
    )
    resource_acces_sepc.add_argument(
        '--resource-readers',
        metavar='RESOURCE_READERS',
        default=[],
        required=False,
        type=arg_parsers.ArgList(),
        help='The set of principals to be granted reader role on the resource. Expected principal formats are user:$userEmail, group:$groupEmail'
        )
    resource_acces_sepc.add_argument(
        '--resource-writers',
        metavar='RESOURCE_WRITERS',
        default=[],
        required=False,
        type=arg_parsers.ArgList(),
        help='The set of principals to be granted writer role on the resource. Expected principal formats are user:$userEmail, group:$groupEmail'
        )
    resource_acces_sepc.add_argument(
        '--resource-owners',
        metavar='RESOURCE_OWNERS',
        default=[],
        required=False,
        type=arg_parsers.ArgList(),
        help='The set of principals to be granted owner role on the resource. Expected principal formats are user:$userEmail, group:$groupEmail'
        )
    dataacces_sepc = parser.add_group(
        required=False,
        help='Specified when applied to data stored on the resource (eg: rows,columns in BigQuery Tables).'
    )
    dataacces_sepc.add_argument(
        '--data-readers',
        metavar='DATA_READERS',
        default=[],
        required=False,
        type=arg_parsers.ArgList(),
        help='The set of principals to be granted reader role on the resource. Expected principal formats are user:$userEmail, group:$groupEmail'
    )
    async_group = parser.add_group(
        mutex=True,
        required=False,
        help='At most one of --async | --validate-only can be specified.')
    async_group.add_argument(
        '--validate-only',
        action='store_true',
        default=False,
        help='Validate the update action, but don\'t actually perform it.')
    base.ASYNC_FLAG.AddToParser(async_group)
    labels_util.AddCreateLabelsFlags(parser)

  @gcloud_exception.CatchHTTPErrorRaiseHTTPException(
      'Status code: {status_code}. {status_message}.')
  def Run(self, args):
    update_mask = data_taxonomy.GenerateAttributeUpdateMask(args)
    if len(update_mask) < 1:
      raise exceptions.HttpException(
          'Update commands must specify at least one additional parameter to change.'
      )

    data_attribute_ref = args.CONCEPTS.data_attribute.Parse()
    dataplex_client = dataplex_util.GetClientInstance()
    update_req_op = dataplex_client.projects_locations_dataTaxonomies_attributes.Patch(
        dataplex_util.GetMessageModule(
        ).DataplexProjectsLocationsDataTaxonomiesAttributesPatchRequest(
            name=data_attribute_ref.RelativeName(),
            updateMask=u','.join(update_mask),
            validateOnly=args.validate_only,
            googleCloudDataplexV1DataAttribute=data_taxonomy
            .GenerateDataAttributeForUpdateRequest(data_attribute_ref, args)))

    validate_only = getattr(args, 'validate_only', False)
    if validate_only:
      log.status.Print('Validation complete.')
      return

    async_ = getattr(args, 'async_', False)
    if not async_:
      response = data_taxonomy.WaitForOperation(update_req_op)
      log.UpdatedResource(data_attribute_ref,
                          details='Operation was successful.')
      return response

    log.status.Print(
        'Updating Data Attribute [{0}] with operation [{1}].'.format(
            data_attribute_ref, update_req_op.name))
    return update_req_op



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utils for GKE Hub Identity Service commands."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.core import exceptions
from googlecloudsdk.core import log
import urllib3

# The max number of auth methods allowed per config.
MAX_AUTH_PROVIDERS = 20


def parse_config(loaded_config, msg):
  """Load FeatureSpec MemberConfig from the parsed ClientConfig CRD yaml file.

  Args:
    loaded_config: YamlConfigFile, The data loaded from the ClientConfig CRD
      yaml file given by the user. YamlConfigFile is from
      googlecloudsdk.command_lib.anthos.common.file_parsers.
    msg: The gkehub messages package.

  Returns:
    member_config: The MemberConfig configuration containing the AuthMethods for
      the IdentityServiceFeatureSpec.
  """

  # Get list of auth providers from ClientConfig.
  if len(loaded_config.data) != 1:
    raise exceptions.Error('Input config file must contain one YAML document.')
  clientconfig = loaded_config.data[0]
  validate_clientconfig_meta(clientconfig)
  auth_providers = clientconfig.GetAuthProviders(name_only=False)

  # Don't accept configs containing more auth providers than MAX_AUTH_PROVIDERS
  auth_providers_count = len(auth_providers)
  if auth_providers_count > MAX_AUTH_PROVIDERS:
    err_msg = ('The provided configuration contains {} identity providers. '
               'The maximum number that can be provided is {}.').format(
                   auth_providers_count, MAX_AUTH_PROVIDERS)
    raise exceptions.Error(err_msg)

  # Create empty MemberConfig and populate it with Auth_Provider configurations.
  member_config = msg.IdentityServiceMembershipSpec()
  # The config must contain at least one auth method
  found_auth_method = False
  for auth_provider in auth_providers:
    # Provision OIDC proto from OIDC ClientConfig dictionary.
    if 'oidc' in auth_provider:
      auth_method = provision_oidc_config(auth_provider, msg)
      member_config.authMethods.append(auth_method)
      found_auth_method = True
    # Provision Google proto from Google ClientConfig dictionary.
    elif 'google' in auth_provider:
      auth_method = provision_google_config(auth_provider, msg)
      member_config.authMethods.append(auth_method)
      found_auth_method = True
    # Provision AzureAD proto from AzureAD ClientConfig dictionary.
    elif 'azureAD' in auth_provider:
      auth_method = provision_azuread_config(auth_provider, msg)
      member_config.authMethods.append(auth_method)
      found_auth_method = True
    # Unsupported configuration found.
    else:
      status_msg = ('Authentication method [{}] is not supported, '
                    'skipping to the next.').format(auth_provider['name'])
      log.status.Print(status_msg)
  if not found_auth_method:
    raise exceptions.Error(
        'No supported authentication method is present in the provided config.')
  return member_config


def validate_clientconfig_meta(clientconfig):
  """Validate the basics of the parsed clientconfig yaml for AIS Hub Feature Spec.

  Args:
    clientconfig: The data field of the YamlConfigFile.
  """

  if 'spec' not in clientconfig:
    raise exceptions.Error('Missing required field .spec')


def provision_oidc_config(auth_method, msg):
  """Provision FeatureSpec OIDCConfig from the parsed yaml file.

  Args:
    auth_method: YamlConfigFile, The data loaded from the yaml file given by the
      user. YamlConfigFile is from
      googlecloudsdk.command_lib.anthos.common.file_parsers.
    msg: The gkehub messages package.

  Returns:
    member_config: A MemberConfig configuration containing a single
      OIDC auth method for the IdentityServiceFeatureSpec.
  """
  auth_method_proto = msg.IdentityServiceAuthMethod()
  auth_method_proto.name = auth_method['name']
  oidc_config = auth_method['oidc']

  # Required Fields.
  if 'issuerURI' not in oidc_config or 'clientID' not in oidc_config:
    raise exceptions.Error(
        'input config file OIDC Config must contain issuerURI and clientID.')
  auth_method_proto.oidcConfig = msg.IdentityServiceOidcConfig()
  auth_method_proto.oidcConfig.issuerUri = oidc_config['issuerURI']
  auth_method_proto.oidcConfig.clientId = oidc_config['clientID']

  validate_issuer_uri(auth_method_proto.oidcConfig.issuerUri,
                      auth_method['name'])

  # Optional Auth Method Fields.
  if 'proxy' in auth_method:
    auth_method_proto.proxy = auth_method['proxy']

  # Optional OIDC Config Fields.
  if 'certificateAuthorityData' in oidc_config:
    auth_method_proto.oidcConfig.certificateAuthorityData = oidc_config[
        'certificateAuthorityData']
  if 'deployCloudConsoleProxy' in oidc_config:
    auth_method_proto.oidcConfig.deployCloudConsoleProxy = oidc_config[
        'deployCloudConsoleProxy']
  if 'extraParams' in oidc_config:
    auth_method_proto.oidcConfig.extraParams = oidc_config['extraParams']
  if 'groupPrefix' in oidc_config:
    auth_method_proto.oidcConfig.groupPrefix = oidc_config['groupPrefix']
  if 'groupsClaim' in oidc_config:
    auth_method_proto.oidcConfig.groupsClaim = oidc_config['groupsClaim']

  # If groupClaim is empty, then groupPrefix should be empty
  if (not auth_method_proto.oidcConfig.groupsClaim and
      auth_method_proto.oidcConfig.groupPrefix):
    raise exceptions.Error(
        'groupPrefix should be empty for method [{}] because groupsClaim is empty.'
        .format(auth_method['name']))

  if 'kubectlRedirectURI' in oidc_config:
    auth_method_proto.oidcConfig.kubectlRedirectUri = oidc_config[
        'kubectlRedirectURI']
  if 'scopes' in oidc_config:
    auth_method_proto.oidcConfig.scopes = oidc_config['scopes']
  if 'userClaim' in oidc_config:
    auth_method_proto.oidcConfig.userClaim = oidc_config['userClaim']
  if 'userPrefix' in oidc_config:
    auth_method_proto.oidcConfig.userPrefix = oidc_config['userPrefix']
  if 'clientSecret' in oidc_config:
    auth_method_proto.oidcConfig.clientSecret = oidc_config['clientSecret']
  if 'enableAccessToken' in oidc_config:
    auth_method_proto.oidcConfig.enableAccessToken = oidc_config[
        'enableAccessToken']
  return auth_method_proto


def provision_google_config(auth_method, msg):
  """Provision FeatureSpec GoogleConfig from the parsed configuration file.

  Args:
    auth_method: YamlConfigFile, The data loaded from the yaml file given by the
      user. YamlConfigFile is from
      googlecloudsdk.command_lib.anthos.common.file_parsers.
    msg: The gkehub messages package.

  Returns:
    member_config: A MemberConfig configuration containing a single Google
    auth method for the IdentityServiceFeatureSpec.
  """
  auth_method_proto = msg.IdentityServiceAuthMethod()
  auth_method_proto.name = auth_method['name']
  google_config = auth_method['google']

  auth_method_proto.googleConfig = msg.IdentityServiceGoogleConfig()

  # Optional Auth Method Fields.
  if 'proxy' in auth_method:
    auth_method_proto.proxy = auth_method['proxy']

  # Required Google Config Fields.
  if 'disable' not in google_config:
    raise exceptions.Error(
        'The "disable" field is not set for the authentication method "{}"'
        .format(auth_method['name']))
  auth_method_proto.googleConfig.disable = google_config['disable']
  return auth_method_proto


def provision_azuread_config(auth_method, msg):
  """Provision FeatureSpec AzureADConfig from the parsed yaml file.

  Args:
    auth_method: YamlConfigFile, The data loaded from the yaml file given by the
      user. YamlConfigFile is from
      googlecloudsdk.command_lib.anthos.common.file_parsers.
    msg: The gkehub messages package.

  Returns:
    member_config: A MemberConfig configuration containing a single
    Azure AD auth method for the IdentityServiceFeatureSpec.
  """
  auth_method_proto = msg.IdentityServiceAuthMethod()
  auth_method_proto.name = auth_method['name']
  auth_method_proto.azureadConfig = msg.IdentityServiceAzureADConfig()
  # Optional Auth Method Fields.
  if 'proxy' in auth_method:
    auth_method_proto.proxy = auth_method['proxy']

  azuread_config = auth_method['azureAD']

  # Required AzureAD Config fields.
  if ('clientID' not in azuread_config or
      'kubectlRedirectURI' not in azuread_config or
      'tenant' not in azuread_config):
    err_msg = ('Authentication method [{}] must contain '
               'clientID, kubectlRedirectURI, and tenant.').format(
                   auth_method['name'])
    raise exceptions.Error(err_msg)
  auth_method_proto.azureadConfig.clientId = azuread_config['clientID']
  auth_method_proto.azureadConfig.kubectlRedirectUri = azuread_config[
      'kubectlRedirectURI']
  auth_method_proto.azureadConfig.tenant = azuread_config['tenant']

  # Optional AzureAD Config fields.
  if 'clientSecret' in azuread_config:
    auth_method_proto.azureadConfig.clientSecret = azuread_config[
        'clientSecret']
  return auth_method_proto


def validate_issuer_uri(issuer_uri, auth_method_name):
  """Validates Issuer URI field of OIDC config.

  Args:
    issuer_uri: issuer uri to be validated
    auth_method_name: auth method name that has this field
  """
  url = urllib3.util.parse_url(issuer_uri)
  if url.scheme != 'https':
    raise exceptions.Error(
        'issuerURI is invalid for method [{}]. Scheme is not https.'.format(
            auth_method_name))
  if url.path is not None and '.well-known/openid-configuration' in url.path:
    raise exceptions.Error(
        'issuerURI is invalid for method [{}]. issuerURI should not contain [{}].'
        .format(auth_method_name, '.well-known/openid-configuration'))
